# NPLIB.pl
# Common routines for NewsPro scripts and addons.
# Version: 3.04


# Load Month/Day Values
@Week_Days = ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
@Months = ('January','February','March','April','May','June','July','August','September','October','November','December');
@Abbrev_Week_Days = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
@Abbrev_Months = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');


# Turn a UNIX date into a human date.
sub GetTheDate {
	# Modified copy of TextClock from http://www.worldwidemart.com/scripts/
	my $thetime = shift;
	unless ($thetime) {
		$thetime = time;
	}
	if ($NPConfig{'TimeOffset'}) {
		$thetime = $thetime + ($NPConfig{'TimeOffset'} * 3600);
	}	
	my $thedate = '';
	($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = (localtime($thetime))[0,1,2,3,4,5,6,8]; 
	$ActualMonth = $Month + 1;
	if ($IsDST == 1) {
	    $Time_Zone = $NPConfig{'Daylight_Time_Zone'};
	}
	else {
    $Time_Zone = $NPConfig{'Standard_Time_Zone'};
	}
	if ($Second < 10) {
	    $Second = "0$Second"; 
	}
	if ($Minute < 10) {
	    $Minute = "0$Minute"; 
		}
	if ($NPConfig{'12HourClock'} == 1) {
		if ($Hour == 0) {
			$Hour = "12";
			$AMPM = "AM";
		}
			elsif ($Hour < 12) {
			$AMPM = "AM";
		}
		elsif ($Hour == 12) {
			$AMPM = "PM";
		}
		else {
			$Hour = $Hour - 12;
			$AMPM = "PM";
		}
	} else {
		if ($Hour < 10) {
		    $Hour = "0$Hour"; 
		}
	}
	if ($Month_Day < 10) {
	    $Month_Day = "0$Month_Day"; 
	}
	$Year = $Year + 1900;
	if ($NPConfig{'Display_Week_Day'}) {
	    $thedate .= "$Week_Days[$Week_Day]";
	    if ($NPConfig{'Display_Month'}) {
	        $thedate .= ", ";
	    }
	}
	if ($NPConfig{'Display_Month'}) {
	    $thedate .= "$Months[$Month] ";
	}
	
	if ($NPConfig{'Display_Month_Day'}) {
	    $thedate .= "$Month_Day";
	    if ($NPConfig{'Display_Year'}) {
	        $thedate .= ", ";
	    }
	}
	if ($NPConfig{'Display_Year'}) {
	    $thedate .= "$Year";
		if ($NPConfig{'Display_Time'}) {
			$thedate .= " -";
		}
	}
	if ($NPConfig{'Display_Time'}) {
	    $thedate .= " ";
	}
	elsif ($NPConfig{'Display_Time_Zone'}) {
	    $thedate .= " ";
	}
	if ($NPConfig{'Display_Time'}) {
	    $thedate .= "$Hour\:$Minute";
		if ($NPConfig{'12HourClock'} == 1) {
			$thedate .= " $AMPM";
		}
	    if ($NPConfig{'Display_Time_Zone'}) {
	        $thedate .= " ";
	    }
	}
	if ($NPConfig{'Display_Time_Zone'}) {
	    $thedate .= "$Time_Zone";
	}
	return $thedate;
	}
	
# DoGMTTime: Return date in GMT text format. Used for HTTP expiry dates
# WARNING: Does not take time into account, always returns time of 00:00:00 (midnight)
sub DoGMTTime {
	my $thetime = shift;
	my $thedate;
	my ($Second,$Minute,$Hour,$Month_Day, $Month,$Year,$Week_Day,$IsDST) = (localtime($thetime))[0,1,2,3,4,5,6,8];	
	if ($Month_Day < 10) {
		$Month_Day = "0$Month_Day";
	}
	$Year = $Year + 1900;
	$thedate = "$Abbrev_Week_Days[$Week_Day], $Month_Day-$Abbrev_Months[$Month]-$Year 00:00:00 GMT";
	return $thedate;
}

# YMDtoUNIX
# Turns a date of form 1999-7-13 (July 13, 1999) into UNIX time.
# Uses time of midnight.
sub YMDtoUNIX {
	my ($textyear, $textmonth, $textday) = @_;
	my $unixtime;
	$textyear = $textyear - 1900;
	$textmonth = $textmonth - 1;
	require Time::Local;
	$unixtime = &Time::Local::timelocal(0,0,0, $textday, $textmonth, $textyear);
	return $unixtime;
}
	
# WriteConfigInfo: Saves %NPConfig settings to npconfig.dat
sub WriteConfigInfo {
	NPopen('NPCFG', ">npconfig.dat");
	if ($npversion) {$NPConfig{'currentversion'} = $npversion;}
	$NPConfig{'firsttime'} = "no";
	foreach $key (keys %NPConfig) {
		$NPConfig{$key} =~ s/^\s+//;
		$NPConfig{$key} =~ s/\s+$//;
		$NPConfig{$key} =~ s/\n//g;
		$NPConfig{$key} =~ s/\r//g;
		print NPCFG "$key``x$NPConfig{$key}\n";
	}
	close(NPCFG);
}

# ReadConfigInfo: Reads configuration from npconfig.dat into %NPConfig
sub ReadConfigInfo {
	NPopen('NPCFG', "npconfig.dat");
	my @npconfig = <NPCFG>;
	close(NPCFG);
	foreach $i (@npconfig) {
		($varname, $varvalue) = split(/``x/, $i);
		$varvalue =~ s/^\s+//;
		$varvalue =~ s/\s+$//;
		$varvalue =~ s/\n//g;
		$NPConfig{$varname} = $varvalue;
	}
}

# GetDirInfo: Gets the absolute path to the current directory.
sub GetDirInfo {
	eval '$home = (getpwuid($< ))[7];';
	if (!-e "C:/") {
	$pwd = `pwd`; chop $pwd;
	}
	if (!eval 'use Cwd;') {
	eval '$cwd = cwd();';
	} else {
	$cwd = "";
	}
	if ($pwd eq "") {
	$dirname = $cwd;
	} else {
	$dirname = $pwd;
	}
}
# GetScriptURL: Gets the full URL of newspro.cgi
sub GetScriptURL {
	local ($ret, $perlwarn);
	$perlwarn = $^W; $^W = 0;
	$ret = 'http://' . $ENV{'SERVER_NAME'} .  
	($ENV{'SERVER_PORT'} != 80 ? ":$ENV{'SERVER_PORT'}" : '') .
	$ENV{'SCRIPT_NAME'};
	$^W = $perlwarn;
	return $ret;
}

# header: A convenience that returns the full Content-type header.
sub header {
	$HeaderPrinted = 1;
	return "Content-type: text/html\n\n";
}

# ReadForm: Reads in POSTed form values to %in
sub ReadForm {
	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
	my @pairs = split(/&/, $buffer);
	foreach $pair (@pairs) {
	($name, $value) = split(/=/, $pair);
		$value =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
		unless($in{$name}) {$in{$name} = $value;}
		else {
			$in{$name} .= "|x|$value";
		}
	}
}


# GetCookies: Reads in HTTP Cookies to %Cookies
sub GetCookies {
	# Load Encode/Decode Data
	@Cookie_Encode_Chars = ('\%', '\+', '\;', '\,', '\=', '\&', '\:\:', '\s');
	
	%Cookie_Encode_Chars = ('\%',   '%25',
	                        '\+',   '%2B',
	                        '\;',   '%3B',
	                        '\,',   '%2C',
	                        '\=',   '%3D',
	                        '\&',   '%26',
	                        '\:\:', '%3A%3A',
	                        '\s',   '+');
	
	@Cookie_Decode_Chars = ('\+', '\%3A\%3A', '\%26', '\%3D', '\%2C', '\%3B', '\%2B', '\%25');
	
	%Cookie_Decode_Chars = ('\+',       ' ',
	                        '\%3A\%3A', '::',
	                        '\%26',     '&',
	                        '\%3D',     '=',
	                        '\%2C',     ',',
	                        '\%3B',     ';',
	                        '\%2B',     '+',
	                        '\%25',     '%');
	local($cookie,$value);
	if ($ENV{'HTTP_COOKIE'}) {
	foreach (split(/; /,$ENV{'HTTP_COOKIE'})) {
		($cookie,$value) = split(/=/);
		foreach $char (@Cookie_Decode_Chars) {
			$cookie =~ s/$char/$Cookie_Decode_Chars{$char}/g;
			$value =~ s/$char/$Cookie_Decode_Chars{$char}/g;
		}
		$Cookies{$cookie} = $value;
	}
	}
}

# SetCookies: Prints a HTTP header, setting any cookies passed to it.
# WARNING: Must be run after GetCookies
sub SetCookies {
    local(@cookies) = @_;
    local($cookie,$value,$char);
	print "Content-type: text/html\n";

    while( ($cookie,$value) = @cookies ) {


        foreach $char (@Cookie_Encode_Chars) {
            $cookie =~ s/$char/$Cookie_Encode_Chars{$char}/g;
            $value =~ s/$char/$Cookie_Encode_Chars{$char}/g;
        }

        print 'Set-Cookie: ' . $cookie . '=' . $value . ';';

    	$exptime = time + 7776000;
		$CookieExpires = DoGMTTime($exptime);
		print ' expires=' . $CookieExpires . ';';
	    print "\n";
        shift(@cookies); shift(@cookies);
    }
	print "\n";
}
sub query_string {
	return $ENV{'QUERY_STRING'};
}

# Is this news item from a different date than the last one?
sub isNewDate {
	my $isNewDate;
	my $mdy = "$Month_Day$Month$Year";
	if ($mdy ne $mdylast || $FileName ne $FileNamelast) {
		$isNewDate = 1;
	} else {
		$isNewDate = 0;
	}
	$mdylast = $mdy;
	$FileNamelast = $FileName;
	return $isNewDate;

}
#######
# NEWS DATABASE LOAD/SAVE
######

# Loads newsdat.txt, and reads news data into a variety of places.
sub loadND {
	undef @NewsData;
	# Load newsdat.txt in @newsdat.
	&NPopen('NEWSDAT', "$NPConfig{'admin_path'}/newsdat.txt");
	my @newsdat = <NEWSDAT>;
	$nd_age = (stat(NEWSDAT))[9];
	close(NEWSDAT);
	
	# Get form field information, add certain hard-coded ones (mainly date & time)
	&SubmitFormFields;
	my @ffields = @formfields;
	@ffield_add = ('newstext', 'newsid', 'newstime', 'newsrand', 'newscat', 'Month', 'Month_Day', 'ActualMonth', 'Week_Day', 'Year', 'Minute', 'Second', 'Hour', 'AMPM', 'Time_Zone');
	if (@Addons_loadND_1) {
		&RunAddons(@Addons_loadND_1);
	}
	push(@ffields, @ffield_add);	
	
	# Declare variables
	my $newsnum = 0;
	my $newsline;
	my $formfield;
	
	# Begin line-by-line processing of newsdat.txt
	foreach $newsline (@newsdat) {
		# Remove ending newline character
		chomp($newsline);
		# Split news item into component variables, using npconfig.pl definition
		&SplitDataFile($newsline);
		# Split the news ID # into component variables.
		($newstime, $newsrand, $newscat) = split(/\,/, $newsid);
		# Generate the full date from the UNIX time
		$newsdate = &GetTheDate($newstime);
		if (@Addons_loadND_2) {
			&RunAddons(@Addons_loadND_2);
		}
		# Go through each defined form field.
		foreach $formfield (@ffields) {
			# Load that particular news variable into the appropriate place in @NewsData
			$NewsData[$newsnum]->{$formfield} = ${$formfield};
		}
		# Establish a hash of news IDs
		$NewsID{$newsid} = $newsnum;
		# Increment $newsnum to continue the processing.
		$newsnum++;
	}
}

# Saves newsdat.txt from @NewsData, which is set by &loadND
sub saveND {
	my $ndentry;
	my $joinline;
	my $ff;
	NPopen('NEWSDAT', ">$NPConfig{'admin_path'}/newsdat.txt");
	foreach $ndentry (@NewsData) {
		$ndentry->{'newsid'} = join(',', $ndentry->{'newstime'}, $ndentry->{'newsrand'}, $ndentry->{'newscat'});
		unless ($ndentry eq "del") {
			foreach $ff (keys %{$ndentry}) { ${$ff} = $ndentry->{$ff}; }
			$joinline = &JoinDataFile;
			$joinline =~ s/\n//g;
			print NEWSDAT "$joinline\n";
		}
	}
	close(NEWSDAT);
}	

# Gets the news variables for a particular item.
sub getNDvar {
	my $nn = shift;
	my $ff;
	foreach $ff (keys %{@NewsData[$nn]}) {
		${$ff} = $NewsData[$nn]->{$ff};
	}
}	

sub HTMLescape {
	my $text = shift;
	$text =~ s/&/&amp;/g;
	$text =~ s/</&lt;/g;
	$text =~ s/>/&gt;/g;
	$text =~ s/"/&quot;/g;
	return $text;
}

sub PrintSelectValues {
	my @values = @_;
	if ($values[0] eq 'same') {
		shift @values;
		foreach $i (@values) {
			print qq~
			<option value="$i">$i</option>~;
		}
	} else {
		my $count = 1;
		foreach $i (@values) {
			print qq~
			<option value="$count">$i</option>~;
			$count++
		}
	}
}
1;